import { InfluxDB } from "@influxdata/influxdb-client";
import * as dotenv from "dotenv";

dotenv.config();
export const INFLUXDB_URL = process.env.INFLUXDB_URL;
export const INFLUXDB_ORG = process.env.INFLUXDB_ORG;
export const INFLUXDB_BUCKET = process.env.INFLUXDB_BUCKET;
export const INFLUXDB_TOKEN = process.env.INFLUXDB_TOKEN;

if (!INFLUXDB_URL || !INFLUXDB_ORG || !INFLUXDB_BUCKET || !INFLUXDB_TOKEN) {
  throw new Error("Missing InfluxDB environment variables");
}

const influxClient = new InfluxDB({
  url: INFLUXDB_URL,
  token: INFLUXDB_TOKEN,
});
export const InfluxWriteAPI = influxClient.getWriteApi(
  INFLUXDB_ORG,
  INFLUXDB_BUCKET,
  "ms"
);
export const InfluxQueryAPI = influxClient.getQueryApi(INFLUXDB_ORG);

require("./services/scoresaber");
require("./services/beatsaver");
require("./api/api");
